﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Speech.Recognition;
using System.Speech.Recognition.SrgsGrammar;
using Palmmedia.AudioStarter.Model;

namespace Palmmedia.AudioStarter
{
    /// <summary>
    /// Responsible for speech recognition.
    /// </summary>
    internal class AudioService
    {
        /// <summary>
        /// The SpeechRecognitionEngine.
        /// </summary>
        private SpeechRecognitionEngine speachRecognizerEngine;

        /// <summary>
        /// The <see cref="DataAccess"/>.
        /// </summary>
        private DataAccess dataAccess;

        /// <summary>
        /// The registered programs.
        /// </summary>
        private IEnumerable<RegisteredProgram> registeredPrograms;

        /// <summary>
        /// Initializes a new instance of the <see cref="AudioService"/> class.
        /// </summary>
        public AudioService()
        {
            this.dataAccess = DataAccess.Instance;
            this.dataAccess.RegisteredProgramsUpdated += new System.EventHandler<System.EventArgs>((s, e) => this.Init());

            this.Init();
        }

        /// <summary>
        /// Initializes the speech engine.
        /// </summary>
        private void Init()
        {
            this.Stop();

            this.speachRecognizerEngine = new SpeechRecognitionEngine();

            this.registeredPrograms = this.dataAccess.GetAllRegisteredPrograms();

            // Add all programs
            SrgsOneOf srgsOneOf = new SrgsOneOf();

            foreach (var program in this.registeredPrograms.Select(p => p.Name.ToLowerInvariant()).Distinct())
            {
                srgsOneOf.Add(new SrgsItem(program));
            }

            // The rule containing the list of programs
            SrgsRule rule = new SrgsRule("programs");
            rule.Add(srgsOneOf);

            // The SrgsDocument containing the rule
            SrgsDocument doc = new SrgsDocument();
            doc.Rules.Add(rule);
            doc.Root = rule;

            // The grammer containing the SrgsDocument
            Grammar grammar = new Grammar(doc);

            // Init SpeechRecognitionEngine
            this.speachRecognizerEngine.SetInputToDefaultAudioDevice();
            this.speachRecognizerEngine.UnloadAllGrammars();
            this.speachRecognizerEngine.LoadGrammar(grammar);
            this.speachRecognizerEngine.SpeechRecognized += new EventHandler<SpeechRecognizedEventArgs>(this.SpeachRecognizerEngine_SpeechRecognized);

            this.speachRecognizerEngine.RecognizeAsync(RecognizeMode.Multiple);
        }

        /// <summary>
        /// Stops the speech engine.
        /// </summary>
        private void Stop()
        {
            if (this.speachRecognizerEngine != null)
            {
                this.speachRecognizerEngine.RecognizeAsyncCancel();
                this.speachRecognizerEngine.SpeechRecognized -= this.SpeachRecognizerEngine_SpeechRecognized;
                this.speachRecognizerEngine.Dispose();
                this.speachRecognizerEngine = null;
            }
        }

        /// <summary>
        /// Handles the SpeechRecognized event of the SpeachRecognizerEngines.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Speech.Recognition.SpeechRecognizedEventArgs"/> instance containing the event data.</param>
        private void SpeachRecognizerEngine_SpeechRecognized(object sender, SpeechRecognizedEventArgs e)
        {
            RegisteredProgram program = this.registeredPrograms.FirstOrDefault(p => p.Name.Equals(e.Result.Text, StringComparison.OrdinalIgnoreCase));

            if (program != null && program.ExecuteableExists)
            {
                Process.Start(program.Path);
            }
        }
    }
}
