﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Xml.Linq;
using Palmmedia.AudioStarter.Model;

namespace Palmmedia.AudioStarter
{
    /// <summary>
    /// Reads and saves <see cref="RegisteredProgram"/> from and to disk.
    /// </summary>
    internal class DataAccess
    {
        /// <summary>
        /// Initializes static members of the <see cref="DataAccess"/> class.
        /// </summary>
        static DataAccess()
        {
            Instance = new DataAccess();
        }

        /// <summary>
        /// Prevents a default instance of the <see cref="DataAccess"/> class from being created.
        /// </summary>
        private DataAccess()
        {
        }

        /// <summary>
        /// Occurs when registered programs are updated.
        /// </summary>
        public event EventHandler<EventArgs> RegisteredProgramsUpdated;

        /// <summary>
        /// Gets the singleton instance of the <see cref="DataAccess"/>.
        /// </summary>
        public static DataAccess Instance { get; private set; }

        /// <summary>
        /// Gets all registered programs.
        /// </summary>
        /// <returns>All registered programs</returns>
        public IEnumerable<RegisteredProgram> GetAllRegisteredPrograms()
        {
            string settingsPath = GetSettingsPath();

            if (!File.Exists(settingsPath))
            {
                return Enumerable.Empty<RegisteredProgram>();
            }

            XDocument document = XDocument.Load(settingsPath);
            var programs = document.Root.Elements("Program");

            return programs.Select(p => new RegisteredProgram() { Name = p.Attribute("name").Value, Path = p.Attribute("path").Value });
        }

        /// <summary>
        /// Saves the registered programms.
        /// </summary>
        /// <param name="programms">The programms to save.</param>
        public void SaveRegisteredProgramms(IEnumerable<RegisteredProgram> programms)
        {
            string settingsPath = GetSettingsPath();

            var programsNode = new XElement("Programs");

            foreach (var program in programms)
            {
                programsNode.Add(
                    new XElement(
                        "Program",
                        new XAttribute("name", program.Name),
                        new XAttribute("path", program.Path)));
            }

            XDocument document = new XDocument(programsNode);
            document.Save(settingsPath);

            this.OnRegisteredProgramsUpdated(EventArgs.Empty);
        }

        /// <summary>
        /// Raises the <see cref="E:RegisteredProgramsUpdated"/> event.
        /// </summary>
        /// <param name="args">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected virtual void OnRegisteredProgramsUpdated(EventArgs args)
        {
            if (this.RegisteredProgramsUpdated != null)
            {
                this.RegisteredProgramsUpdated(this, args);
            }
        }

        /// <summary>
        /// Gets the settings path.
        /// Creates the corresponding directory if it does not exist yet.
        /// </summary>
        /// <returns>The settings path</returns>
        private static string GetSettingsPath()
        {
            string settingsDirectory = System.Environment.GetFolderPath(System.Environment.SpecialFolder.ApplicationData);
            settingsDirectory = Path.Combine(settingsDirectory, "Palmmedia", "Audiostarter");

            if (!Directory.Exists(settingsDirectory))
            {
                Directory.CreateDirectory(settingsDirectory);
            }

            return Path.Combine(settingsDirectory, "RegisteredPrograms.xml");
        }
    }
}
