﻿using System;
using System.ComponentModel;
using System.Diagnostics;
using System.IO;

namespace Palmmedia.AudioStarter.Model
{
    /// <summary>
    /// Represents a program that can be started by speech.
    /// </summary>
    internal class RegisteredProgram : IDataErrorInfo
    {
        /// <summary>
        /// Gets or sets the name which has to be spoken to start the program.
        /// </summary>
        public string Name { get; set; }

        /// <summary>
        /// Gets or sets the path to the executable.
        /// </summary>
        public string Path { get; set; }

        /// <summary>
        /// Gets a value indicating whether the executable exists or not.
        /// </summary>
        public bool ExecuteableExists
        {
            get
            {
                return File.Exists(this.Path);
            }
        }

        #region IDataErrorInfo Members

        /// <summary>
        /// Gets an error message indicating what is wrong with this object.
        /// </summary>
        /// <value></value>
        /// <returns>An error message indicating what is wrong with this object. The default is an empty string ("").</returns>
        public string Error
        {
            get
            {
                return null;
            }
        }

        /// <summary>
        /// Gets the <see cref="System.String"/> with the specified property name.
        /// </summary>
        public string this[string propertyName]
        {
            get
            {
                return this.GetValidationError(propertyName);
            }
        }

        #endregion // IDataErrorInfo Members

        #region Validation

        /// <summary>
        /// The properties that are includes in validation.
        /// </summary>
        private static readonly string[] ValidatedProperties = 
        { 
            "Name", 
            "Path"
        };

        /// <summary>
        /// Gets a value indicating whether this instance is valid.
        /// </summary>
        /// <value><c>true</c> if this instance is valid; otherwise, <c>false</c>.</value>
        public bool IsValid
        {
            get
            {
                foreach (string property in ValidatedProperties)
                {
                    if (this.GetValidationError(property) != null)
                    {
                        return false;
                    }
                }

                return true;
            }
        }

        /// <summary>
        /// Gets the validation error.
        /// </summary>
        /// <param name="propertyName">Name of the property.</param>
        /// <returns><c>null</c> if no error exists, otherwise the error.</returns>
        private string GetValidationError(string propertyName)
        {
            if (Array.IndexOf(ValidatedProperties, propertyName) < 0)
            {
                return null;
            }

            string error = null;

            switch (propertyName)
            {
                case "Name":
                    error = this.ValidateName();
                    break;

                case "Path":
                    error = this.ValidatePath();
                    break;

                default:
                    Debug.Fail("Unexpected property being validated: " + propertyName);
                    break;
            }

            return error;
        }

        /// <summary>
        /// Validates the name.
        /// </summary>
        /// <returns>The error.</returns>
        private string ValidateName()
        {
            if (IsStringMissing(this.Name))
            {
                return "Enter name";
            }

            return null;
        }

        /// <summary>
        /// Validates the path.
        /// </summary>
        /// <returns>The error.</returns>
        private string ValidatePath()
        {
            if (IsStringMissing(this.Path))
            {
                return "Enter path";
            }

            if (!this.ExecuteableExists)
            {
                return "Executeable does not exist";
            }

            return null;
        }

        /// <summary>
        /// Determines whether the string is empty or not.
        /// </summary>
        /// <param name="value">The value.</param>
        /// <returns>
        /// 	<c>true</c> if empty; otherwise, <c>false</c>.
        /// </returns>
        private static bool IsStringMissing(string value)
        {
            return string.IsNullOrEmpty(value) || value.Trim() == string.Empty;
        }

        #endregion
    }
}
