﻿using System.Collections.ObjectModel;
using System.Linq;
using System.Windows.Input;
using Palmmedia.AudioStarter.Model;

namespace Palmmedia.AudioStarter.ViewModel
{
    /// <summary>
    /// The ViewModel for the MainWindow.
    /// </summary>
    internal class MainWindowViewModel : ViewModelBase
    {
        #region Fields

        /// <summary>
        /// Determines whether saving changes is available.
        /// </summary>
        private bool saveAvailable;

        /// <summary>
        /// Indicates whether a registered program is currently edited.
        /// </summary>
        private bool editingProgram;

        /// <summary>
        /// The <see cref="DataAccess"/>.
        /// </summary>
        private DataAccess dataAccess;

        #endregion

        /// <summary>
        /// Initializes a new instance of the <see cref="MainWindowViewModel"/> class.
        /// </summary>
        public MainWindowViewModel()
        {
            this.dataAccess = DataAccess.Instance;

            this.RegisteredPrograms = new ObservableCollection<RegisteredProgramViewModel>();
            this.ProgramToCreate = new RegisteredProgramViewModel(new RegisteredProgram());

            this.AddCommand = new RelayCommand(param => this.Add(), param => this.ProgramToCreate.IsValid);
            this.EditCommand = new RelayCommand(param => this.Edit(), param => this.SelectedProgram != null);
            this.DeleteCommand = new RelayCommand(param => this.Delete(), param => this.SelectedProgram != null);
            this.SaveCommand = new RelayCommand(param => this.Save(), param => this.saveAvailable);

            this.LoadRegisteredPrograms();
        }

        #region Properties

        /// <summary>
        /// Gets the command adds a new program.
        /// </summary>
        public ICommand AddCommand { get; private set; }

        /// <summary>
        /// Gets the command edits an existing program.
        /// </summary>
        public ICommand EditCommand { get; private set; }

        /// <summary>
        /// Gets the command deletes an existing program.
        /// </summary>
        public ICommand DeleteCommand { get; private set; }

        /// <summary>
        /// Gets the command saves all changes.
        /// </summary>
        public ICommand SaveCommand { get; private set; }

        /// <summary>
        /// Gets all registered programs.
        /// </summary>
        public ObservableCollection<RegisteredProgramViewModel> RegisteredPrograms { get; private set; }

        /// <summary>
        /// The registered program which is currently created / updated.
        /// </summary>
        private RegisteredProgramViewModel programToCreate;

        /// <summary>
        /// Gets or sets the registered program which is currently created / updated.
        /// </summary>
        public RegisteredProgramViewModel ProgramToCreate
        {
            get
            {
                return this.programToCreate;
            }

            set
            {
                this.programToCreate = value;
                this.OnPropertyChanged("ProgramToCreate");
            }
        }

        /// <summary>
        /// The registered program which is currently selected.
        /// </summary>
        private RegisteredProgramViewModel selectedProgram;

        /// <summary>
        /// Gets or sets the registered program which is currently selected.
        /// </summary>
        public RegisteredProgramViewModel SelectedProgram
        {
            get
            {
                return this.selectedProgram;
            }

            set
            {
                this.selectedProgram = value;
                this.OnPropertyChanged("SelectedProgram");
            }
        }

        #endregion

        /// <summary>
        /// Adds or updates the currently created / edited program.
        /// </summary>
        private void Add()
        {
            if (this.editingProgram)
            {
                this.SelectedProgram.Name = this.ProgramToCreate.Name;
                this.SelectedProgram.Path = this.ProgramToCreate.Path;
            }
            else
            {
                this.RegisteredPrograms.Add(this.ProgramToCreate);
            }

            this.ProgramToCreate = new RegisteredProgramViewModel(new RegisteredProgram());
            this.editingProgram = false;
            this.saveAvailable = true;
        }

        /// <summary>
        /// Edits the currently selected program.
        /// </summary>
        private void Edit()
        {
            this.editingProgram = true;
            this.ProgramToCreate.Name = this.SelectedProgram.Name;
            this.ProgramToCreate.Path = this.SelectedProgram.Path;
        }

        /// <summary>
        /// Deletes the currently selected program.
        /// </summary>
        private void Delete()
        {
            this.RegisteredPrograms.Remove(this.SelectedProgram);
            this.saveAvailable = true;
        }

        /// <summary>
        /// Saves all changes.
        /// </summary>
        private void Save()
        {
            this.dataAccess.SaveRegisteredProgramms(this.RegisteredPrograms.Select(r => r.RegisteredProgram));
            this.saveAvailable = false;
        }

        /// <summary>
        /// Loads the registered programs.
        /// </summary>
        private void LoadRegisteredPrograms()
        {
            this.RegisteredPrograms.Clear();

            var programs = this.dataAccess.GetAllRegisteredPrograms();

            foreach (var program in programs)
            {
                this.RegisteredPrograms.Add(new RegisteredProgramViewModel(program));
            }
        }
    }
}
